dtw_loss_functions.soft_dtw_implementations.soft_dtw_cuda_ron.autograd module — dtw_loss_functions documentation

# dtw_loss_functions.soft_dtw_implementations.soft_dtw_cuda_ron.autograd module

class dtw_loss_functions.soft_dtw_implementations.soft_dtw_cuda_ron.autograd.SoftDTWAutograd(*args, **kwargs)

  Bases: `Function`

  Methods

  `backward`(ctx, grad_output) | Define a formula for differentiating the operation with backward mode automatic differentiation.
  `forward`(ctx, D, gamma, bandwidth) | Define the forward of the custom autograd Function.

  static backward(ctx, grad_output: Tensor)

    Define a formula for differentiating the operation with backward mode automatic differentiation.

    This function is to be overridden by all subclasses. (Defining this function is equivalent to defining the `vjp` function.)

    It must accept a context `ctx` as the first argument, followed by as many outputs as the `forward()` returned (None will be passed in for non tensor outputs of the forward function), and it should return as many tensors, as there were inputs to `forward()`. Each argument is the gradient w.r.t the given output, and each returned value should be the gradient w.r.t. the corresponding input. If an input is not a Tensor or is a Tensor not requiring grads, you can just pass None as a gradient for that input.

    The context can be used to retrieve tensors saved during the forward pass. It also has an attribute `ctx.needs_input_grad` as a tuple of booleans representing whether each input needs gradient. E.g., `backward()` will have `ctx.needs_input_grad[0] = True` if the first input to `forward()` needs gradient computed w.r.t. the output.

  static forward(ctx, D: Tensor, gamma: float, bandwidth: float | None)

    Define the forward of the custom autograd Function.

    This function is to be overridden by all subclasses. There are two ways to define forward:

    Usage 1 (Combined forward and ctx):

        @staticmethod
        def forward(ctx: Any, *args: Any, **kwargs: Any) -> Any:
            pass

    - It must accept a context ctx as the first argument, followed by any number of arguments (tensors or other types).

    - See combining-forward-context for more details

    Usage 2 (Separate forward and ctx):

        @staticmethod
        def forward(*args: Any, **kwargs: Any) -> Any:
            pass

        @staticmethod
        def setup_context(ctx: Any, inputs: Tuple[Any, ...], output: Any) -> None:
            pass

    - The forward no longer accepts a ctx argument.

    - Instead, you must also override the `torch.autograd.Function.setup_context()` staticmethod to handle setting up the `ctx` object. `output` is the output of the forward, `inputs` are a Tuple of inputs to the forward.

    - See extending-autograd for more details

    The context can be used to store arbitrary data that can be then retrieved during the backward pass. Tensors should not be stored directly on ctx (though this is not currently enforced for backward compatibility). Instead, tensors should be saved either with `ctx.save_for_backward()` if they are intended to be used in `backward` (equivalently, `vjp`) or `ctx.save_for_forward()` if they are intended to be used for in `jvp`.